/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-08-01 15:17:10.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core.config;

import cn.com.qmth.examcloud.app.model.Constants;
import cn.com.qmth.examcloud.app.model.LoginInfo;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.model.UserInfo;
import cn.com.qmth.examcloud.app.service.UserAuthService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.*;
import javax.servlet.FilterConfig;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Date;

import static cn.com.qmth.examcloud.app.model.Constants.PLATFORM_SESSION_EXPIRE_TIME;

/**
 * 认证信息过滤器
 *
 * @author: fengdesheng
 * @since: 2018/8/1
 */
public class TokenFilter implements Filter {
    private final static Logger log = LoggerFactory.getLogger(TokenFilter.class);
    private UserAuthService userAuthService;

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        log.debug("init");
    }

    @Override
    public void destroy() {
        log.debug("destroy");
    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {
        CustomHttpServletRequest customRequest = this.initCustomRequest(servletRequest);
        if (customRequest != null) {
            filterChain.doFilter(customRequest, servletResponse);
            return;
        }
        filterChain.doFilter(servletRequest, servletResponse);
    }

    private CustomHttpServletRequest initCustomRequest(ServletRequest servletRequest) {
        //用户带的token实际为缓存的redisKey
        HttpServletRequest request = (HttpServletRequest) servletRequest;
        String redisKey = request.getHeader(Constants.PARAM_TOKEN);
        if (StringUtils.isBlank(redisKey)) {
            //token为空，则不用过滤处理
            return null;
        }

        //通过redisKey获取原始的登录信息
        LoginInfo loginInfo = userAuthService.getLoginInfo(redisKey);
        if (loginInfo == null) {
            //loginInfo为空，则代表尚未登录不用处理
            return null;
        }

        //处理登录过的信息
        CustomHttpServletRequest customRequest = new CustomHttpServletRequest(request);
        if (!this.filterAccessUrl(request.getServletPath())) {
            //判断原始的login token是否在有效时间内
            if (loginInfo.hasExpired(PLATFORM_SESSION_EXPIRE_TIME)) {
                try {
                    //已超过有效时间，则自动登录续期
                    Result<UserInfo> result = userAuthService.login(loginInfo);
                    UserInfo userInfo = result.getData();
                    if (userInfo != null) {
                        //登录成功后缓存新的login token
                        log.info("redisKey:" + redisKey + " key:" + userInfo.getKey() + " newToken:" + userInfo.getToken());
                        loginInfo.setToken(userInfo.getToken());
                        loginInfo.setCreateTime(new Date());
                        userAuthService.cacheLoginInfo(loginInfo, redisKey);
                    }
                } catch (Exception e) {
                    log.error(e.getMessage());
                }
            }
        }

        //替换原始token到request请求中
        customRequest.addHeader(Constants.PARAM_TOKEN, loginInfo.getToken());

        //初始化内部接口请求鉴权
        userAuthService.initRequestTrace();
        return customRequest;
    }

    private boolean filterAccessUrl(String url) {
        //跳过不用处理的访问地址
        if (url.contains("/user/logout")) {
            return true;
        }
        return false;
    }

    public void setUserAuthService(UserAuthService userAuthService) {
        this.userAuthService = userAuthService;
    }

}